/**
 * This class is an example class that show the functionalities that spring controller can do.
 * You can copy the functions and use in you class.
 */
package vt.CS5934.SwitchRoom.controllers;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import vt.CS5934.SwitchRoom.models.ExampleModel;
import vt.CS5934.SwitchRoom.models.ResponseModel;

/**
 * The "@RestController" made the class into rest handle class
 * The "@RequestMapping("example")" on the class level make it only react to url ".../example/..."
 */
@RestController
@RequestMapping("example")
public class ExampleController {

    /**
     * You can use logger.[trace,debug,info,warn,error]("messages") to log into file
     */
    Logger logger = LoggerFactory.getLogger(ExampleController.class);

    /**
     * This function handles GET request on url "/example"
     * @return a ResponseModel in String format
     */
    @GetMapping
    public String getExample() {
        logger.info("You reached the getExample() function.");

        return ResponseModel.generateResponse(
                "This this a example String response",
                HttpStatus.OK,
                "Some Json Object Later");
    }

    /**
     * This function handles GET request on url "/example/<a-long-val>"
     * @param userId this variable will be passed in the url path
     * @return a ResponseModel in String format
     */
    @GetMapping("/{userId}")
    public String getExampleWithInput(@PathVariable long userId){
        return ResponseModel.generateResponse(
                "This this a example String response",
                HttpStatus.OK,
                "You GET this function with id = "+ userId);
    }

    /**
     * This function handles GET request on url "/example/detail/<a-long-val>"
     * @param userId this variable will be passed in the url path
     * @return a ResponseModel in String format
     */
    @GetMapping("/detail/{userId}")
    public String getExampleDetailWithInput(@PathVariable long userId){
        return ResponseModel.generateResponse(
                "This this a example String response",
                HttpStatus.OK,
                "You GET this function with detail id: "+ userId);
    }

    /**
     * This function handles GET request on url "/example/getUser"
     * @return an example user info in the data field
     */
    @RequestMapping("/getUser")
    public String getExampleUser(){
        ExampleModel user = new ExampleModel(233, "example-user");
        return ResponseModel.generateResponse(
                "There is your example user info",
                HttpStatus.OK,
                user);
    }


    /**
     * This function will handle the post function and change the JSON body into data class
     * @param newUser the JSON input in the request body
     */
    @PostMapping("/newUser")
    public String handlePost(@RequestBody ExampleModel newUser){
        logger.info("You reached the handlePost() function.");
        return ResponseModel.generateResponse(
                "I received your new user data, check it in data section",
                HttpStatus.OK,
                newUser);
    }

    /**
     * If you need to delete anything from DB, use DELETE request
     * @param userId user id
     * @return info
     */
    @DeleteMapping("/logout/{userId}")
    public String logout(@PathVariable long userId){
        logger.warn("You reached the logout() function.");
        return ResponseModel.generateResponse(
                "I received your logout request",
                HttpStatus.OK,
                null);
    }
}

/*
Change logs:
Date        |       Author          |   Description
2022-10-11  |    Fangzheng Zhang    |    create class and init
 */
